Unit ExtLBox;

{------------------------------------------------------------------------------
  ExtLBox ist eine Listbox-Komponente fuer Delphi. Sie entspricht der normalen
  Listbox, kennt aber ein neues zusaetzliches Ereignis "OnChange". Wenn ein an-
  derer Eintrag in der Listbox mit der Maus oder Tastatur selektiert wird, dann
  wird dieses Ereignis aufgerufen. Das Ereignis wird ebenfalls aufgerufen, wenn
  ein anderer Eintrag in der Listbox "ferngesteuert" durch einem Aufruf der Art:
    ListBox.ItemIndex:= Nummer;
  selektiert wird.

  Die Komponente ist Freeware. Vielleicht kann sie jemand gebrauchen.
  Installation: Komponente muss in den Komponenten-Pfad kopiert werden, z.Bsp.
  C:\DELPHI\LIB. Dann mit Menue "Optionen" -> "Komponente installieren" ->
  "Hinzufuegen" die ExtLBox in Delphi regstrieren.

  Jan - M. Strube, Stuttgart
  E-Mail: 100333.2744@CompuServe.Com
 ------------------------------------------------------------------------------}

interface

uses
  WinProcs, Messages, Classes, StdCtrls;

type
  TExtListBox = Class(TListBox)
    Private
      FLastSel: Integer;
      FOnChange: TNotifyEvent;
      procedure Click; override;
      function GetItemIndex: Integer;
      procedure SetItemIndex(Value: Integer);
    Protected
      procedure Change; Virtual;
    Published
      property OnChange: TNotifyEvent read FOnChange write FOnChange;
    Public
      constructor Create(AOwner: TComponent); override;
      procedure ResyncAfterItemLostSelection;
      property ItemIndex: Integer read GetItemIndex write SetItemIndex;
    end;

procedure Register;

implementation

constructor TExtListBox.Create;
begin
  Inherited Create(AOwner);
  FLastSel:= -1;
end;

procedure TExtListBox.Change;
begin
  FLastSel:= ItemIndex;
  if assigned(FOnChange) then FOnChange(self);
end;

procedure TExtListBox.Click;
begin
  inherited Click;
  if FLastSel <> ItemIndex then Change;
end;

function TExtListBox.GetItemIndex: Integer;
begin
  Result:= SendMessage(Handle, LB_GETCURSEL, 0, 0);
end;

procedure TExtListBox.SetItemIndex(Value: Integer);
VAR
  LastIndexPos: LongInt;
begin
  LastIndexPos:= GetItemIndex;
  if LastIndexPos <> Value then SendMessage(Handle, LB_SETCURSEL, Value, 0);
  if GetItemIndex <> LastIndexPos then Change;
end;

procedure TExtListBox.ResyncAfterItemLostSelection;
begin
  FLastSel:= -1;
end;

procedure Register;
begin
  RegisterComponents('Standard', [TExtListBox]);
end;

end.

